#pragma once

// AronMacros (copied from Config.h)
/************************************
 * TYPES ****************************
 ***********************************/
#define HANDLE_PRIMITIVE_TYPES \
    RUN_ARON_MACRO(Int, int, INT) \
    RUN_ARON_MACRO(Long, long, LONG) \
    RUN_ARON_MACRO(Float, float, FLOAT) \
    RUN_ARON_MACRO(Double, double, DOUBLE) \
    RUN_ARON_MACRO(String, string, STRING) \
    RUN_ARON_MACRO(Bool, bool, BOOL) \
    RUN_ARON_MACRO(Time, time, TIME)

/************************************
 * DATA *****************************
 ***********************************/
#define HANDLE_PRIMITIVE_DATA \
    RUN_ARON_MACRO(Int, int, INT) \
    RUN_ARON_MACRO(Long, long, LONG) \
    RUN_ARON_MACRO(Float, float, FLOAT) \
    RUN_ARON_MACRO(Double, double, DOUBLE) \
    RUN_ARON_MACRO(String, string, STRING) \
    RUN_ARON_MACRO(Bool, bool, BOOL)

#define ARON_VERSION "beta 0.2.2"


module armarx
{
    module aron
    {

        const string VERSION = ARON_VERSION;

        /*************************
         * General Definitions ***
         ************************/
        sequence<byte> AronByteSequence;
        sequence<int> AronIntSequence;
        sequence<string> AronStringSequence;
        dictionary<string, int> AronIntDict;


        /*************************
         * Aron Types ************
         ************************/
        module type
        {
            enum Maybe
            {
                eNone,
                eOptional,
                eRawPointer,
                eUniquePointer,
                eSharedPointer
            };

            class AronType {
                //string VERSION = ARON_VERSION;
                Maybe maybe = Maybe::eNone; // Every type can have a maye type flag or not
            }
            sequence<AronType> AronTypeList;
            dictionary<string, AronType> AronTypeDict;

            // Container types
            class AronList extends AronType { AronType acceptedType; }
            class AronTuple extends AronType { AronTypeList elementTypes; }
            class AronPair extends AronType { AronType acceptedType1; AronType acceptedType2; }
            class AronObject extends AronType { AronObject parent; string objectName; AronTypeDict elementTypes; }
            class AronDict extends AronType { AronType acceptedType; }

            // Complex Types (serialize to ndarray)


            class AronNDArray extends AronType { AronIntSequence dimensions; int elementSize; string typeName; }
            // NdArray should look like this:
            // class AronNDArray extends AronType { int ndim; string type; }
            // type can be: "uint8", "int8", "uint16", "int16", "uint32", "int32", "float32", "float64"



            class AronEigenMatrix extends AronType { int rows; int cols; string typeName; }
            class AronEigenQuaternion extends AronType { string typeName; }

            // pixelType can be: "rgb24", "depth32"
            class AronImage extends AronType { string pixelType; }

            // to be removed:
            class AronIVTCByteImage extends AronType { }
            class AronOpenCVMat extends AronType { AronIntSequence shape; string typeName; }

            class AronPCLPointCloud extends AronType { string typeName; }
            class AronPosition extends AronType { }
            class AronOrientation extends AronType { }
            class AronPose extends AronType { }

            // Enum types
            class AronIntEnum extends AronType { string enumName; AronIntDict acceptedValues; }

            // Primitive Types
            #define RUN_ARON_MACRO(upperType, lowerType, capsType) \
                class Aron##upperType extends AronType { };

                    HANDLE_PRIMITIVE_TYPES
            #undef RUN_ARON_MACRO
        };


        /*************************
         * Aron Data *************
         ************************/
        module data
        {
            class AronData {
                //string VERSION = ARON_VERSION;
            };
            sequence<AronData> AronDataList;
            dictionary<string, AronData> AronDataDict;

            // Container Data
            class AronList extends AronData { AronDataList elements; };
            class AronDict extends AronData { AronDataDict elements; };

            // Complex Data. The NDArray contains more or less the same information as an AronType, but there is no other way to do it
            // Especially, note the difference between the type's typeName (e.g. "GRAY_SCALE_IMAGE") and the data's type ("0")
            // Further, note the difference between the type's dimensions (e.g. 128x128) and the data's dimensions (128x128x3 for RGB)
            class AronNDArray extends AronData { AronIntSequence dimensions; string type; AronByteSequence data; };

            // NDArray should look like this: (type is be as above and implies the bit-width of an element)
            // class AronNDArray extends AronData { AronIntSequence shape; string type; AronByteSequence data; };


            // Primitive Data
            #define RUN_ARON_MACRO(upperType, lowerType, capsType) \
                class Aron##upperType extends AronData { lowerType value; };

                    HANDLE_PRIMITIVE_DATA
            #undef RUN_ARON_MACRO

            // useful for memory ice_conversions
            sequence<AronDict> AronDictSeq;
        };

    };
};

#undef HANDLE_PRIMITIVE_DATA
#undef HANDLE_PRIMITIVE_TYPES
