#pragma once

#include "EntityInstanceBase.h"
#include "EntitySnapshotBase.h"
#include "EntityBase.h"
#include "ProviderSegmentBase.h"
#include "CoreSegmentBase.h"
#include "MemoryBase.h"

#include <RobotAPI/libraries/armem/core/ice_conversions_templates.h>
#include <RobotAPI/libraries/armem/core/ice_conversions.h>

#include <RobotAPI/libraries/aron/core/navigator/type/forward_declarations.h>

#include <RobotAPI/interface/armem/memory.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::armem::base::detail
{
    void toIceItem(data::detail::MemoryItem& ice, const MemoryItem& item);
    void fromIceItem(const data::detail::MemoryItem& ice, MemoryItem& item);

    void toIce(aron::data::AronDictPtr& ice, const aron::datanavigator::DictNavigatorPtr& bo);
    void fromIce(const aron::data::AronDictPtr& ice, aron::datanavigator::DictNavigatorPtr& bo);

    void toIce(aron::type::AronTypePtr& ice, const aron::typenavigator::ObjectNavigatorPtr& bo);
    void fromIce(const aron::type::AronTypePtr& ice, aron::typenavigator::ObjectNavigatorPtr& bo);
}
namespace armarx::armem::base
{
    void toIce(data::EntityInstanceMetadata& ice, const EntityInstanceMetadata& metadata);
    void fromIce(const data::EntityInstanceMetadata& ice, EntityInstanceMetadata& metadata);

    void toIce(data::EntityInstanceMetadataPtr& ice, const EntityInstanceMetadata& metadata);
    void fromIce(const data::EntityInstanceMetadataPtr& ice, EntityInstanceMetadata& metadata);


    template <class ...Args>
    void toIce(data::EntityInstance& ice, const EntityInstanceBase<Args...>& data)
    {
        detail::toIceItem(ice, data);

        // detail::toIce(std::ref(ice.data).get(), data.data());
        detail::toIce(ice.data, data.data());
        toIce(ice.metadata, data.metadata());
    }
    template <class ...Args>
    void fromIce(const data::EntityInstance& ice, EntityInstanceBase<Args...>& data)
    {
        detail::fromIceItem(ice, data);

        detail::fromIce(ice.data, data.data());
        fromIce(ice.metadata, data.metadata());
    }

    template <class ...Args>
    void toIce(data::EntitySnapshot& ice, const EntitySnapshotBase<Args...>& snapshot)
    {
        detail::toIceItem(ice, snapshot);

        ice.instances.clear();
        snapshot.forEachInstance([&ice](const auto & instance)
        {
            armem::toIce(ice.instances.emplace_back(), instance);
        });
    }
    template <class ...Args>
    void fromIce(const data::EntitySnapshot& ice, EntitySnapshotBase<Args...>& snapshot)
    {
        detail::fromIceItem(ice, snapshot);

        snapshot.clear();
        for (const data::EntityInstancePtr& iceInstance : ice.instances)
        {
            snapshot.addInstance(armem::fromIce<typename EntitySnapshotBase<Args...>::EntityInstanceT>(iceInstance));
        }
    }

    template <class ...Args>
    void toIce(data::Entity& ice, const EntityBase<Args...>& entity)
    {
        detail::toIceItem(ice, entity);

        ice.history.clear();
        entity.forEachSnapshot([&ice](const auto & snapshot)
        {
            armem::toIce(ice.history[armem::toIce<long>(snapshot.time())], snapshot);
        });
    }
    template <class ...Args>
    void fromIce(const data::Entity& ice, EntityBase<Args...>& entity)
    {
        detail::fromIceItem(ice, entity);

        entity.clear();
        for (const auto& [key, snapshot] : ice.history)
        {
            entity.addSnapshot(armem::fromIce<typename EntityBase<Args...>::EntitySnapshotT>(snapshot));
        }
    }


    template <class ...Args>
    void toIce(data::ProviderSegment& ice, const ProviderSegmentBase<Args...>& providerSegment)
    {
        detail::toIceItem(ice, providerSegment);

        detail::toIce(ice.aronType, providerSegment.aronType());
        ARMARX_CHECK(!providerSegment.aronType() || ice.aronType);

        ice.entities.clear();
        providerSegment.forEachEntity([&ice](const auto & entity)
        {
            armem::toIce(ice.entities[entity.name()], entity);
        });
    }
    template <class ...Args>
    void fromIce(const data::ProviderSegment& ice, ProviderSegmentBase<Args...>& providerSegment)
    {
        detail::fromIceItem(ice, providerSegment);

        detail::fromIce(ice.aronType, providerSegment.aronType());
        ARMARX_CHECK(!providerSegment.aronType() || ice.aronType);

        providerSegment.clear();
        for (const auto& [key, entity] : ice.entities)
        {
            providerSegment.addEntity(
                armem::fromIce<typename ProviderSegmentBase<Args...>::EntityT>(entity));
        }
    }

    template <class ...Args>
    void toIce(data::CoreSegment& ice, const CoreSegmentBase<Args...>& coreSegment)
    {
        detail::toIceItem(ice, coreSegment);

        detail::toIce(ice.aronType, coreSegment.aronType());
        ARMARX_CHECK(!coreSegment.aronType() || ice.aronType);

        ice.providerSegments.clear();
        coreSegment.forEachProviderSegment([&ice](const auto & providerSegment)
        {
            armem::toIce(ice.providerSegments[providerSegment.name()], providerSegment);
        });
    }
    template <class ...Args>
    void fromIce(const data::CoreSegment& ice, CoreSegmentBase<Args...>& coreSegment)
    {
        detail::fromIceItem(ice, coreSegment);

        detail::fromIce(ice.aronType, coreSegment.aronType());
        ARMARX_CHECK(!coreSegment.aronType() || ice.aronType);

        // fromIce(ice.providerSegments, coreSegment.providerSegments());
        coreSegment.clear();
        for (const auto& [key, providerSegment] : ice.providerSegments)
        {
            coreSegment.addProviderSegment(
                armem::fromIce<typename CoreSegmentBase<Args...>::ProviderSegmentT>(providerSegment));
        }
    }

    template <class ...Args>
    void toIce(data::Memory& ice, const MemoryBase<Args...>& memory)
    {
        base::detail::toIceItem(ice, memory);

        ice.coreSegments.clear();
        memory.forEachCoreSegment([&ice](const auto & coreSegment)
        {
            armem::toIce(ice.coreSegments[coreSegment.name()], coreSegment);
        });
    }
    template <class ...Args>
    void fromIce(const data::Memory& ice, MemoryBase<Args...>& memory)
    {
        base::detail::fromIceItem(ice, memory);

        memory.clear();
        for (const auto& [key, coreSegment] : ice.coreSegments)
        {
            // We can avoid using CoreSegment's copy constructor this way:
            armem::fromIce(coreSegment, memory.addCoreSegment(coreSegment->id.coreSegmentName));
        }
    }

}
