#pragma once

#include <string>
#include <mutex>
#include <map>
#include <memory>
#include <sstream>

#include <bsoncxx/json.hpp>
#include <mongocxx/client.hpp>
#include <mongocxx/pool.hpp>
#include <mongocxx/stdx.hpp>
#include <mongocxx/uri.hpp>
#include <mongocxx/instance.hpp>
#include <bsoncxx/builder/stream/helpers.hpp>
#include <bsoncxx/builder/stream/document.hpp>
#include <bsoncxx/builder/stream/array.hpp>


namespace armarx::armem::server::ltm::mongodb
{

    using PoolClientPtr = mongocxx::pool::entry;

    /**
     * @brief A manager of multiple mongodb connection
     */
    class ConnectionManager
    {
    public:
        struct MongoDBSettings
        {
            std::string host = "localhost";
            unsigned int port = 25276;
            std::string user = "";
            std::string password = "";
            std::string database = "Test";
            int minPoolSize = 5;
            int maxPoolSize = 100;


            bool isSet() const
            {
                // we always need a host and a port
                return !host.empty() and port != 0;
            }

            std::string baseUri() const
            {
                std::stringstream ss;
                ss << "mongodb://";

                if (!user.empty())
                {
                    ss << user;
                    if (!password.empty())
                    {
                        ss << ":" << password;
                    }
                    ss << "@";
                }
                ss << host;
                return ss.str();
            }

            std::string key() const
            {
                // TODO: What happens if a connection exists and you would like to open another one with a different user (e.g. that sees different things)?
                return "mongodb://" + host + ":" + std::to_string(port);
            }

            std::string uri() const
            {
                return baseUri() + ":" + std::to_string(port) + "/?minPoolSize=" + std::to_string(minPoolSize) + "&maxPoolSize=" + std::to_string(maxPoolSize);
            }

            std::string toString() const
            {
                return uri() + "&database=" + database;
            }
        };

        static mongocxx::pool& Connect(const MongoDBSettings& settings);
        static bool ConnectionIsValid(const MongoDBSettings& settings, bool forceNewConnection = false);
        static bool ConnectionExists(const MongoDBSettings& settings);

    private:
        static void initialize_if();


    private:
        static std::mutex initializationMutex;
        static bool initialized;
        static std::map<std::string, std::unique_ptr<mongocxx::pool>> Connections;

    };
}
