#pragma once

#include <sstream>
#include <string>

#include <Eigen/Core>

#include <RobotAPI/libraries/aron/core/navigator/visitors/TypedDataVisitor.h>


namespace armarx::aron
{

    class TypedDataDisplayVisitor : public aron::visitor::TypedDataVisitor
    {
    public:

        static std::string getValue(TypeNavigator& type, DataNavigator& data);


    public:

        TypedDataDisplayVisitor();


        std::stringstream value;


        bool visitEnter(DictTypeNavigator&, DictDataNavigator& data) override;
        bool visitEnter(ObjectTypeNavigator&, DictDataNavigator& data) override;

        bool visitEnter(ListTypeNavigator&, ListDataNavigator& data) override;
        bool visitEnter(TupleTypeNavigator&, ListDataNavigator& data) override;


        bool visit(BoolTypeNavigator&, BoolDataNavigator& data) override;
        bool visit(DoubleTypeNavigator&, DoubleDataNavigator& data) override;
        bool visit(FloatTypeNavigator&, FloatDataNavigator& data) override;
        bool visit(IntTypeNavigator&, IntDataNavigator& data) override;
        bool visit(LongTypeNavigator&, LongDataNavigator& data) override;
        bool visit(StringTypeNavigator&, StringDataNavigator& data) override;
        bool visit(TimeTypeNavigator&, LongDataNavigator& data) override;


        bool visit(EigenMatrixTypeNavigator&, NDArrayDataNavigator& data) override;
        bool visit(EigenQuaternionTypeNavigator&, NDArrayDataNavigator& data) override;
        bool visit(ImageNavigator&, NDArrayDataNavigator& data) override;
        bool visit(IVTCByteImageTypeNavigator&, NDArrayDataNavigator& data) override;
        bool visit(OpenCVMatTypeNavigator&, NDArrayDataNavigator& data) override;
        bool visit(PCLPointCloudTypeNavigator&, NDArrayDataNavigator& data) override;
        bool visit(PoseTypeNavigator&, NDArrayDataNavigator& data) override;
        bool visit(PositionTypeNavigator&, NDArrayDataNavigator& data) override;
        bool visit(OrientationTypeNavigator&, NDArrayDataNavigator& data) override;


    protected:

        std::string coeffSep;
        Eigen::IOFormat eigenIof;


    private:

        template <typename ScalarT>
        void processEigenMatrix(EigenMatrixTypeNavigator&, NDArrayDataNavigator& data);
        void processEigenQuaternion(NDArrayDataNavigator& data);

        void setStreamPrecision();
        void setStreamPrecision(std::ostream& os);

    };

}
