/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "IntEnum.h"

#include <SimoxUtility/meta/type_name.h>


namespace armarx::aron::cppserializer::serializer
{
    // constructors
    IntEnumSerializer::IntEnumSerializer(const typenavigator::IntEnumNavigatorPtr& e) :
        detail::SerializerBase<typenavigator::IntEnumNavigator, IntEnumSerializer>(
            e->getEnumName(),
            simox::meta::get_type_name<data::AronNDArray>(),
            simox::meta::get_type_name<type::AronIntEnum>(),
            e)
    {
    }

    CppBlockPtr IntEnumSerializer::getResetSoftBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        std::string resolved_accessor = ResolveMaybeAccessor(accessor, typenavigator);

        block_if_data->addLine(accessor + nextEl() + "resetSoft();");
        return ResolveMaybeResetSoftBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr IntEnumSerializer::getWriteTypeBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine(accessor + "::writeType(w, " + MAYBE_TO_STRING(typenavigator->getMaybe()) + ");");
        return b;
    }

    CppBlockPtr IntEnumSerializer::getResetHardBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        if (typenavigator->getMaybe() != type::Maybe::eNone)
        {
            b->addLine(accessor + " = {};");
        }
        else
        {
            b->addLine(accessor + nextEl() + "resetHard();");
        }
        return b;
    }

    CppBlockPtr IntEnumSerializer::getWriteBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = CppBlockPtr(new CppBlock());
        block_if_data->addLine(accessor + nextEl() + "write(w);");

        return ResolveMaybeWriteBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr IntEnumSerializer::getReadBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine(accessor + nextEl() + "read(r);");
        return b;
    }

    CppBlockPtr IntEnumSerializer::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine("if (not (" + accessor + " == " + otherInstanceAccessor + "))");
        b->addLineAsBlock("return false;");
        return b;
    }
}
