/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Rainer Kartmann (rainer dot kartmann at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/libraries/aron/core/codegenerator/codeWriter/cpp/serializer/detail/NDArraySerializerBase.h>
#include <RobotAPI/libraries/aron/core/navigator/type/ndarray/Image.h>

#include <map>
#include <memory>
#include <string>


namespace armarx::aron::cppserializer::serializer
{

    class ImageSerializer :
        virtual public detail::NDArraySerializerBase<typenavigator::ImageNavigator, ImageSerializer>
    {
    public:
        using PointerType = std::shared_ptr<ImageSerializer>;

    public:

        ImageSerializer(const typenavigator::ImageNavigatorPtr& n);

        // virtual implementations
        virtual CppBlockPtr getResetHardBlock(const std::string& accessor) const override;
        virtual CppBlockPtr getResetSoftBlock(const std::string& accessor) const override;
        virtual CppBlockPtr getWriteTypeBlock(const std::string&) const override;
        virtual CppBlockPtr getWriteBlock(const std::string&) const override;
        virtual CppBlockPtr getReadBlock(const std::string&) const override;
        virtual CppBlockPtr getEqualsBlock(const std::string&, const std::string&) const override;


    private:

        static constexpr const char* ITERATOR_ACCESSOR = "_iterator";
        static constexpr const char* SHAPE_ACCESSOR = "_shape";
        static constexpr const char* NDIM_ACCESSOR = "_ndim";

        /// Maps ARON pixel types to OpenCV image type constants.
        static const std::map<typenavigator::ImagePixelType, std::string> pixelTypeMap;

    };

}
