/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/libraries/aron/core/codegenerator/codeWriter/cpp/serializer/detail/SerializerBase.h>
#include <RobotAPI/libraries/aron/core/navigator/type/enum/IntEnum.h>

#include <map>
#include <utility>  // std::pair


namespace armarx::aron::cppserializer::serializer
{

    class IntEnumClassSerializer;
    typedef std::shared_ptr<IntEnumClassSerializer> IntEnumClassSerializerPtr;

    class IntEnumClassSerializer :
        virtual public detail::SerializerBase<typenavigator::IntEnumNavigator, IntEnumClassSerializer>
    {
    public:
        // constructors
        IntEnumClassSerializer(const typenavigator::IntEnumNavigatorPtr&);

        // virtual implementations
        std::vector<CppFieldPtr> getPublicVariableDeclarations(const std::string&) const;
        virtual CppBlockPtr getResetHardBlock(const std::string&) const override;
        virtual CppBlockPtr getResetSoftBlock(const std::string&) const override;
        virtual CppBlockPtr getWriteTypeBlock(const std::string&) const override;
        virtual CppBlockPtr getWriteBlock(const std::string&) const override;
        virtual CppBlockPtr getReadBlock(const std::string&) const override;
        virtual CppBlockPtr getEqualsBlock(const std::string&, const std::string&) const override;

        // TODO: Move some of those methods to upper class for enums (if we want to support multiple enums)
        CppCtorPtr toCopyCtor(const std::string&) const;
        CppCtorPtr toInnerEnumCtor(const std::string&) const;
        CppEnumPtr toInnerEnumDefinition() const;
        CppMethodPtr toIntMethod() const;
        CppMethodPtr toCopyAssignmentMethod() const;
        CppMethodPtr toEnumAssignmentMethod() const;
        CppMethodPtr toIntAssignmentMethod() const;
        CppMethodPtr toToStringMethod() const;
        CppMethodPtr toFromStringMethod() const;

    private:
        // Members
        static const std::map<std::string, std::pair<std::string, int>> ACCEPTED_TYPES;
        static constexpr const char* IMPL_ENUM = "__ImplEnum";
    };
}
