/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD / STL
#include <memory>
#include <stack>
#include <sstream>

// Base Class
#include <RobotAPI/libraries/aron/core/io/typeIO/Writer.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/Concepts.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/writer/nlohmannJSON/NlohmannJSONWriterToken.h>

namespace armarx::aron::typeIO::writer
{
    class NlohmannJSONWriter :
        virtual public WriterInterface
    {
    public:
        NlohmannJSONWriter() = default;

        virtual void writeStartObject(const WriteStartObjectInput&) override;
        virtual void writeEndObject() override;
        virtual void writeStartList(const WriteStartListInput&) override;
        virtual void writeEndList() override;
        virtual void writeStartDict(const WriteStartDictInput&) override;
        virtual void writeEndDict() override;
        virtual void writeStartTuple(const WriteStartTupleInput&) override;
        virtual void writeEndTuple() override;
        virtual void writeStartPair(const WriteStartPairInput&) override;
        virtual void writeEndPair() override;

        virtual void writeEigenMatrix(const WriteEigenMatrixInput&) override;
        virtual void writeEigenQuaternion(const WriteEigenQuaternionInput&) override;
        virtual void writeImage(const WriteImageInput&) override;
        virtual void writeIVTCByteImage(const WriteIVTCByteImageInput&) override;
        virtual void writeOpenCVMat(const WriteOpenCVMatInput&) override;
        virtual void writePCLPointCloud(const WritePCLPointCloudInput&) override;
        virtual void writePosition(const WritePositionInput&) override;
        virtual void writeOrientation(const WriteOrientationInput&) override;
        virtual void writePose(const WritePoseInput&) override;

        virtual void writeInt(const WritePrimitiveInput&) override;
        virtual void writeLong(const WritePrimitiveInput&) override;
        virtual void writeFloat(const WritePrimitiveInput&) override;
        virtual void writeDouble(const WritePrimitiveInput&) override;
        virtual void writeString(const WritePrimitiveInput&) override;
        virtual void writeBool(const WritePrimitiveInput&) override;
        virtual void writeTime(const WritePrimitiveInput&) override;

        virtual void writeKey(const std::string&) override;
        virtual void writeExtends() override;

        nlohmann::json getResult() const
        {
            return lastRemovedToken->getElement();
        }

    private:
        bool wroteInitialStartObject = false;
        NlohmannJSONWriterTokenPtr lastRemovedToken = nullptr;
        std::stack<NlohmannJSONWriterTokenPtr> stack = {};

        Path generatePath();
    };
}
