/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <unordered_map>

// Base class
#include "../Navigator.h"

// ArmarX

namespace armarx::aron::typenavigator::detail
{
    template<typename AronTypeT, typename DerivedT>
    class NavigatorBase :
        virtual public typenavigator::Navigator
    {
    public:
        using PointerType = std::shared_ptr<DerivedT>;
        using AronType = AronTypeT;

    public:
        NavigatorBase() :
            aron(new AronTypeT())
        {
        }

        NavigatorBase(const typename AronType::PointerType& o) :
            aron(new AronTypeT(*o))
        {
            ARMARX_CHECK_NOT_NULL(aron);
        }

        // operators
        operator typename AronType::PointerType()
        {
            return aron;
        }

        virtual bool operator==(const Navigator& other) const override
        {
            const auto& n = DerivedT::DynamicCast(other);
            return *this == n;
        }
        virtual bool operator==(const NavigatorPtr& other) const override
        {
            if (!other)
            {
                return false;
            }
            return *this == *other;
        }

        virtual bool operator==(const DerivedT&) const = 0;
        bool operator==(const PointerType& other) const
        {
            if (other)
            {
                return *this == *other;
            }
            return false;
        }

        // virtual implementations
        virtual type::AronTypePtr toAronPtr() const override
        {
            ARMARX_CHECK_NOT_NULL(aron);
            return aron;
        }
        void setMaybe(const type::Maybe m) override
        {
            aron->maybe = m;
        }

        type::Maybe getMaybe() const override
        {
            return aron->maybe;
        }

        // static methods
        static PointerType DynamicCast(const NavigatorPtr& n)
        {
            return std::dynamic_pointer_cast<DerivedT>(n);
        }

        static DerivedT& DynamicCast(Navigator& n)
        {
            return dynamic_cast<DerivedT&>(n);
        }

        static const DerivedT& DynamicCast(const Navigator& n)
        {
            return dynamic_cast<const DerivedT&>(n);
        }

        static PointerType DynamicCastAndCheck(const NavigatorPtr& n)
        {
            ARMARX_CHECK_NOT_NULL(n);
            auto casted = DerivedT::DynamicCast(n);
            ARMARX_CHECK_NOT_NULL(casted);
            return casted;
        }

    protected:
        typename AronType::PointerType aron;
    };
}
