/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base class
#include "../detail/NDArrayNavigatorBase.h"

namespace armarx::aron::typenavigator
{
    class EigenQuaternionNavigator;
    typedef std::shared_ptr<EigenQuaternionNavigator> EigenQuaternionNavigatorPtr;

    class EigenQuaternionNavigator :
        virtual public detail::NDArrayNavigatorBase<type::AronEigenQuaternion, EigenQuaternionNavigator>
    {
    public:
        // constructors
        EigenQuaternionNavigator(const Path& path = Path());
        EigenQuaternionNavigator(const type::AronEigenQuaternionPtr&, const Path& path);

        // operators
        virtual bool operator==(const EigenQuaternionNavigator&) const override;

        // public member functions
        std::string checkTypename(const std::string&) const;

        std::string getTypename() const;

        void setTypename(const std::string&);

        type::AronEigenQuaternionPtr toAronEigenQuaternionPtr() const;

        // virtual implementations
        virtual std::string getName() const override;

    public:
        const std::map<std::string, std::vector<std::string>> ACCEPTED_TYPES =
        {
            {"float", {}},
            {"double", {}}
        };
        const std::vector<int> ACCEPTED_DIMENSION = {1, 4};
    };
}
