/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Rainer Kartmann (rainer dot kartmann at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/libraries/aron/core/navigator/type/detail/NDArrayNavigatorBase.h>

#include <memory>
#include <string>


namespace armarx::aron::typenavigator
{
    enum class ImagePixelType
    {
        /// 3 (8-bit) byte channels for R, G, and B.
        Rgb24,   // ==> CV_8UC1
        /// 1 (32-bit) float channel for depth.
        Depth32, // ==> CV_32FC1
    };


    class ImageNavigator :
        virtual public detail::NDArrayNavigatorBase<type::AronImage, ImageNavigator>
    {
    public:

        ImageNavigator(const Path& path = {});
        ImageNavigator(const type::AronImagePtr&, const Path& path);


        type::AronImagePtr toAronImagePtr() const;

        /// Get the pixel type.
        ImagePixelType getPixelType() const;
        void setPixelType(const ImagePixelType& type) const;
        void setPixelType(const std::string& typeName) const;


        // virtual implementations
        virtual std::string getName() const override;


        // operators
        virtual bool operator==(const ImageNavigator&) const override;


    public:

        static std::string pixelTypeToName(ImagePixelType type);
        static ImagePixelType pixelTypeFromName(const std::string& name);
        static std::set<ImagePixelType> pixelTypes();
        static std::set<std::string> pixelTypeNames();

    };

    using ImageNavigatorPtr = std::shared_ptr<ImageNavigator>;

}
